use colored::Colorize; // For coloring the output
use std::collections::HashMap;

use crate::Coordinates;
use crate::DEBUG_MODE;

// Output the map based on map vector
pub fn output_map(
	map: &Vec<Vec<char>>, 
	blocks: &HashMap<char, Vec<([u8; 3],bool, &str)>>,
	player_coordinates: &Coordinates, 
)
{
	let (screen_width, screen_height) = termion::terminal_size().unwrap();
	let mut distance_you_can_see: [[i16; 2]; 2] =                                                                                               
	[
		[
			(player_coordinates.x as f32 - (screen_width as f32 / 2.0)) as i16,
			(player_coordinates.x as f32 + (screen_width as f32 / 2.0)) as i16,
		],
		[
			(player_coordinates.z as f32 - (screen_height as f32 / 2.0)) as i16,
			(player_coordinates.z as f32 + (screen_height as f32 / 2.0)) as i16,
		],
	];

	// Dynamic sizing of output, I forgot how this works, don't change pls
	// X	
	if distance_you_can_see[0][0] < 0
	{
		distance_you_can_see[0][1] -= distance_you_can_see[0][0];
		distance_you_can_see[0][0] = 0;
	}
	if distance_you_can_see[0][1] > map[player_coordinates.z as usize].len() as i16
	{
		distance_you_can_see[0][0] -= map[player_coordinates.z as usize].len() as i16 - distance_you_can_see[0][1];
		distance_you_can_see[0][1] = map[player_coordinates.z as usize].len() as i16;
	}
	// Z
	if distance_you_can_see[1][0] < 0
	{
		distance_you_can_see[1][1] -= distance_you_can_see[1][0];
		distance_you_can_see[1][0] = 0;
	}
	if distance_you_can_see[1][1] > map.len() as i16
	{
		distance_you_can_see[1][0] -= map.len() as i16 - distance_you_can_see[1][1];
		distance_you_can_see[1][1] = map.len() as i16;
	}

	// Output debug info
	if DEBUG_MODE == true 
	{
		println!("{:?}", player_coordinates);
		println!("Screen Width: {}, Screen Height: {}", screen_width, screen_height);
		println!("{:?}", distance_you_can_see);
	}
	

	// Loop over array and print each block
	for z in distance_you_can_see[1][0]..distance_you_can_see[1][1] 
	{
		if z >= 0 && z < map.len() as i16 
		{
			for x in distance_you_can_see[0][0]..distance_you_can_see[0][1]
			{
				if x >= 0 && x < map[z as usize].len() as i16 
				{
					let character: char = map[z as usize][x as usize];
					// Check if the character is at the current coordinates
					if z == player_coordinates.z && x == player_coordinates.x 
					{
						print!
						(
							"{}",
							" "
							.on_truecolor(40,235,181)
							.truecolor(0,0,0)
						)
					}
					// Else print what is below (the ground)
					else if blocks.contains_key(&character) 
					{
						print!
						(
							"{}",
							character // Output's the current block
							.to_string()
							.truecolor(
								blocks.get(&character).unwrap()[0].0[0],
								blocks.get(&character).unwrap()[0].0[1],
								blocks.get(&character).unwrap()[0].0[2]
							)
						)
					}
					// If the character is invalid print a null value
					else 
					{
						print!
						(
							"{}",
							"X"
							.to_string()
							.truecolor(
								blocks.get(&'X').unwrap()[0].0[0],
								blocks.get(&'X').unwrap()[0].0[1],
								blocks.get(&'X').unwrap()[0].0[2]
							)
						)
					}
					
				}
			}
			println!("");
		}
	}
}
