use std::fs; // For reading the map and save files
use crate::Coordinates;

// This function reads the map file and puts it into a 2d vector of integers, each integer 
// Refers to a block (see top comment) and the function returns this vector
pub fn parse_map(player_coordinates: &Coordinates) -> Result<Vec<Vec<char>>, u8> 
{
	let mut map: Vec<Vec<char>> = Vec::new(); // Initialises the Ground map vector
	let map_file = format!("data/{}.map",player_coordinates.map);

	// Read the ground map file
	if let Ok(parsed_map_file_contents) = fs::read_to_string(&map_file) 
	{
		let map_file_contents = parsed_map_file_contents;
		let map_rows = map_file_contents.split("\n").collect::<Vec<_>>(); // Split the file contents into rows

		// Loop over each row to parse each column of it
		for row in 0..map_rows.len() 
		{
			// Turn each column into an integer and collect it into a row vector
			let current_row = map_rows[row].chars().collect();
			map.push(current_row); // Then push this vector to the full map vector
		}
	}
	else 
	{
		eprintln!("Error: Could not read file: {}", map_file);
		return Err(1)
	}

	return Ok(map)
}
